#include "sim_data.hpp"
#include "create_grid.hpp"
#include <cmath>

SmoothInjection::SmoothInjection
(double D,
 double eta,
 double v_w,
 double GM,
 const PhysicalGeometry1D& pg):
  D_(D),
  eta_(eta),
  v_w_(v_w),
  GM_(GM),
  pg_(pg) {}

Conserved SmoothInjection::operator()
  (vector<double> const& vertices,
   vector<Primitive> const& cells,
   size_t point,
   double /*t*/,
   double /*dt*/) const
{
  const double volume = pg_.calcVolume(vertices.at(point+1))-
    pg_.calcVolume(vertices.at(point));
  const double mass = cells.at(point).Density*volume;
  const double r = 0.5*(vertices.at(point)+vertices.at(point+1));
  const double u = cells.at(point).Velocity.x;
  const double p = cells.at(point).Pressure;
  const Conserved mass_injection(D_*pow(r,-eta_)*volume,
				 Vector2D(0,0),
				 D_*pow(r,-eta_)*volume*pow(v_w_,2)/2);
  const Conserved gravity(0,
			  Vector2D(-1,0)*GM_*mass/pow(r,2),
			  -GM_*mass*u/pow(r,2));
  const Conserved geometry(0,
			   2*Vector2D(volume*(p/r),0),
			   0);
  return mass_injection+gravity+geometry;
}

Conserved Ratchet::CalcFlux(vector<double> const& Vertices,
			    vector<Primitive> const& Cells,
			    RiemannSolver const& rs, 
			    vector<double> const& vertex_velocity,
			    int i) const
{
  if(i==0){
    Primitive ghost = Cells[0];
    ghost.Velocity.x = -abs(ghost.Velocity.x);
    ghost.Density /= 10;
    ghost.Pressure /= 10;
    double vv = vertex_velocity[0];
    return rs(ghost,Cells[0],vv);
  }
  else if(i==static_cast<int>(Vertices.size())-1){
    Primitive ghost = Cells[Vertices.size()-2];
    ghost.Velocity.x = abs(ghost.Velocity.x);
    ghost.Density /= 10;
    ghost.Pressure /= 10;
    double vv = vertex_velocity[Vertices.size()-1];
    return rs(Cells[Vertices.size()-2],ghost,vv);
  }
  else
    throw UniversalError("Index inside bulk of grid");
}

namespace {
  vector<double> logspace
  (double start,
   double end,
   size_t n)
  {
    vector<double> res(n);
    const double len = static_cast<double>(n);
    for(size_t i=0;i<n;i++){
      const double id = static_cast<double>(i);
      const double expnt = start + (end-start)*id/len;
      res.at(i) = pow(10,expnt);
    }
    return res;
  }
}

SimData::SimData(void):
  pg_(),
  sr_(),
  eos_(5./3.),
  rs_(),
  vm_(),
  bc_(),
  source_(1,2.5,1,1,pg_),
  grid_(logspace(-1.5,2.5,200)), 
  sim_(pg_,
       grid_,
       sr_,
       Uniform(1e-9),
       Uniform(1e-3),
       Uniform(0),
       Uniform(0),
       eos_,
       rs_,
       vm_,
       bc_,
       source_)
{
  assert(grid_.size()>1);
}

hdsim1D& SimData::getSim(void)
{
  return sim_;
}
